@@ -45,8 +45,8 @@ class HTSFullLabel(UserList):
             Falseのときは d, f における休符の長さ情報が維持される。
         """
         # 休符周辺の仕様をSinsyに近づける。
-        new_label = adjust_syllables_to_hts(self)
-        new_label = adjust_notes_to_hts(new_label, strict=strict_sinsy_style)
+        new_label = adjust_syllables_to_sinsy(self)
+        new_label = adjust_notes_to_sinsy(new_label, strict=strict_sinsy_style)
         # 文字列にする
         s = '\n'.join([str(oneline) for oneline in new_label])
         # ファイル出力
@@ -163,6 +163,7 @@ class HTSFullLabel(UserList):
                     onelines.append(ol)
         self.data = onelines
         self._fill_phonemes()
+        self._fill_note_position()
         return self
 
     def _fill_phonemes(self):
@@ -178,6 +179,36 @@ class HTSFullLabel(UserList):
             ol.after_next_phoneme = extended_self[i + 2].phoneme
         return self
 
+    def _fill_note_position(self):
+        """
+        「Phrase内で何番目か」の項を埋める。
+        Phraseの扱いは難しいので、「休符からの距離」を代わりに用いる。
+        """
+        counter = 'xx'
+        # 直前の休符からの距離を数える
+        for note in self.song.all_notes:
+            # 休符のときは距離をリセット
+            if note.is_pau():
+                note.position = 'xx'
+                counter = 0
+            # 休符ではないけれど休符位置が分からないときは未登録のまま
+            elif counter == 'xx':
+                continue
+            # 休符ではなく、休符位置が分かっているときは登録
+            else:
+                counter += 1
+                note.position = counter
+        # 次の休符までの距離を登録する。
+        for note in reversed(self.song.all_notes):
+            if note.is_pau():
+                note.position_backward = 'xx'
+                counter = 0
+            elif counter == 'xx':
+                continue
+            else:
+                counter += 1
+                note.position_backward = counter
+
     def generate_songobj(self):
         """
         フルコンテキストラベルの情報をもとにSongオブジェクトをつくる。
@@ -307,22 +338,23 @@ class OneLine:
 
     @p.setter
     def p(self, phoneme_contexts: list):
-        self.phoneme.language_independent_identity = phoneme_contexts[0]
-        self.before_previous_phoneme.identity = phoneme_contexts[1]
-        self.previous_phoneme.identity = phoneme_contexts[2]
-        self.phoneme.identity = phoneme_contexts[3]
-        self.next_phoneme.identity = phoneme_contexts[4]
-        self.after_next_phoneme.identity = phoneme_contexts[5]
-        self.before_previous_phoneme.flag = phoneme_contexts[6]
-        self.previous_phoneme.flag = phoneme_contexts[7]
-        self.phoneme.flag = phoneme_contexts[8]
-        self.next_phoneme.flag = phoneme_contexts[9]
-        self.after_next_phoneme.flag = phoneme_contexts[10]
-        self.phoneme.position = phoneme_contexts[11]
-        self.phoneme.position_backward = phoneme_contexts[12]
-        self.phoneme.distance_from_previous_vowel = phoneme_contexts[13]
-        self.phoneme.distance_to_next_vowel = phoneme_contexts[14]
-        self.phoneme.undefined_context = phoneme_contexts[15]
+        (self.phoneme.language_independent_identity,
+         self.before_previous_phoneme.identity,
+         self.previous_phoneme.identity,
+         self.phoneme.identity,
+         self.next_phoneme.identity,
+         self.after_next_phoneme.identity,
+         self.before_previous_phoneme.flag,
+         self.previous_phoneme.flag,
+         self.phoneme.flag,
+         self.next_phoneme.flag,
+         self.after_next_phoneme.flag,
+         self.phoneme.position,
+         self.phoneme.position_backward,
+         self.phoneme.distance_from_previous_vowel,
+         self.phoneme.distance_to_next_vowel,
+         self.phoneme.undefined_context
+         ) = phoneme_contexts[0:16]
 
     @property
     def a(self) -> list:
@@ -500,19 +532,6 @@ class Song(UserList):
         リストとしての要素数と、コンテキストに記載されている要素数が一致するか点検する。
         Noteに記載されている
         """
-        # 各要素数がちゃんとしているか確認する。
-        # 楽曲内フレーズ数が、ラベルに記載されている値と一致するか確認する。
-        # assert len(self) == self.number_of_phrases, \
-        #     f'楽曲内フレーズ数に不整合があります。len(Song):{len(self)}, ラベル記載値: {self.number_of_phrases}'
-        # フレーズ内の値をチェック
-        # for phrase in self.all_phrases:
-        #     syllables = list(chain.from_iterable(phrase))
-        #     # 各フレーズ内音節数が、ラベルに記載されている値と一致するか確認する。
-        #     assert len(syllables) == phrase.number_of_syllables, \
-        #         f'フレーズ内音節数に不整合があります。{len(syllables)}, {phrase.number_of_syllables}'
-        #     # 各フレーズ内音素数が、ラベルに記載されている値と一致するか確認する。
-        #     assert sum(list(map(len, syllables))) == phrase.number_of_phonemes, \
-        #         'フレーズ内音素数に不整合があります。'
         # 各音節内音素数が、ラベルに記載されている値と一致するか確認する。
         for i, syllable in enumerate(self.all_syllables):
             assert str(len(syllable)) == str(syllable.number_of_phonemes), \
@@ -593,7 +612,7 @@ class Note(UserList):
     @property
     def position(self):
         """
-        フレーズ内での位置
+        フレーズ内での位置(e18)
         """
         return self.contexts[17]
 
@@ -601,10 +620,22 @@ class Note(UserList):
     def position(self, position: int):
         self.contexts[17] = position
 
+    @property
+    def position_backward(self):
+        """
+        フレーズ内での後ろから数えた位置(e19)
+        1-indexed
+        """
+        return int(self.contexts[18])
+
+    @position_backward.setter
+    def position_backward(self, position_backward: int):
+        self.contexts[18] = position_backward
+
     @property
     def number_of_syllables(self):
         """
-        ノート内音節数
+        ノート内音節数(e6)
         """
         return self.contexts[5]
 
@@ -635,7 +666,8 @@ class Syllable(UserList):
     @property
     def position(self):
         """
-        ノート内での位置
+        ノート内での位置(b2)
+        1-indexed
         """
         return int(self.contexts[1])
 
@@ -643,6 +675,18 @@ class Syllable(UserList):
     def position(self, position: int):
         self.contexts[1] = position
 
+    @property
+    def position_backward(self):
+        """
+        ノート内での後ろから数えた位置(b3)
+        1-indexed
+        """
+        return int(self.contexts[2])
+
+    @position_backward.setter
+    def position_backward(self, position_backward: int):
+        self.contexts[2] = position_backward
+
     @property
     def number_of_phonemes(self):
         """
@@ -687,27 +731,22 @@ class Phoneme:
         return f'{self.start} {self.end} {self.identity}'
 
 
-def adjust_syllables_to_hts(full_label: HTSFullLabel) -> HTSFullLabel:
+def adjust_syllables_to_sinsy(full_label: HTSFullLabel) -> HTSFullLabel:
     """
     出力用に休符まわりの音節コンテキストを調整する。
     """
-    new_label = HTSFullLabel([deepcopy(ol) for ol in full_label])
-
+    new_label = deepcopy(full_label)
     for ol in new_label[1:]:
         if ol.previous_syllable[0].identity in ('pau', 'sil'):
-            ol.a[0] = 'xx'
-            ol.a[1] = 'xx'
-            ol.a[2] = 'xx'
+            ol.a[0:2] = ['xx'] * 3
     for ol in new_label[:-1]:
         if ol.next_syllable[0].identity in ('pau', 'sil'):
-            ol.c[0] = 'xx'
-            ol.c[1] = 'xx'
-            ol.c[2] = 'xx'
+            ol.c[0:2] = ['xx'] * 3
 
     return new_label
 
 
-def adjust_notes_to_hts(full_label: HTSFullLabel, strict=True) -> HTSFullLabel:
+def adjust_notes_to_sinsy(full_label: HTSFullLabel, strict=True) -> HTSFullLabel:
     """
     休符の前後のノート情報を調整したい。
     Sinsy仕様の d, f では休符を飛ばした音符を検出して処理している。
@@ -715,22 +754,30 @@ def adjust_notes_to_hts(full_label: HTSFullLabel, strict=True) -> HTSFullLabel:
     音程関連は必ずやる。
     休符の長さに関する前後情報は持っておきたい。
     """
-    new_label = HTSFullLabel([deepcopy(ol) for ol in full_label])
-
-    for ol in new_label[1:]:
-        if ol.previous_note.is_pau():
-            ol.d[0:3] = ['xx'] * 3
-            if strict:
+    new_label = deepcopy(full_label)
+    if strict:
+        # 前のノートに関する処理
+        for ol in new_label[1:]:
+            if ol.previous_note.is_pau():
+                ol.d[0:3] = ['xx'] * 3
                 ol.d[3:8] = ['xx'] * 5
-
+        # 次のノートに関する処理
+        for ol in new_label[:-1]:
+            if ol.next_note.is_pau():
+                ol.f[0:3] = ['xx'] * 3
+                ol.f[3:8] = ['xx'] * 5
+    else:
+        # 前のノートに関する処理
+        for ol in new_label[1:]:
+            if ol.previous_note.is_pau():
+                ol.d[0:3] = ['xx'] * 3
+        # 次のノートに関する処理
+        for ol in new_label[:-1]:
+            if ol.next_note.is_pau():
+                ol.f[0:3] = ['xx'] * 3
+    # 現在のノートに関する処理
     for ol in new_label:
         if ol.note.is_pau():
             ol.e[0:2] = ['xx'] * 2
 
-    for ol in new_label[:-1]:
-        if ol.next_note.is_pau():
-            ol.f[0:3] = ['xx'] * 3
-            if strict:
-                ol.f[3:8] = ['xx'] * 5
-
     return new_label
