@@ -4,27 +4,13 @@
 USTファイルとデータを扱うモジュールです。
 """
 import re
-from collections import UserDict, UserList
+from collections import UserDict
 from copy import deepcopy
-from functools import lru_cache
+# from functools import lru_cache
+# from pprint import pprint
+from typing import List
 
 
-def main():
-    """実行されたときの挙動"""
-    print('デフォ子かわいいよデフォ子\n')
-
-    print('ust読み取りテストをします。')
-    path = input('ustのパスを入力してください。\n>>> ')
-    ust = load(path)
-    print()
-
-    for note in ust.values:
-        print(note)
-
-    input('\nPress Enter to exit.')
-
-
-@lru_cache()
 def notenum_as_abc(notenum) -> str:
     """
     音階番号をABC表記に変更する(C1=24, C4=)
@@ -48,153 +34,162 @@ def notenum_as_abc(notenum) -> str:
     return d[str(notenum)]
 
 
-def load(path: str, mode='r', encoding='shift-jis'):
+def load(path: str, encoding: str = 'shift-jis'):
     """
     USTを読み取り
     """
-    # USTを文字列として取得
-    try:
-        with open(path, mode=mode, encoding=encoding) as f:
-            s = f.read()
-    except UnicodeDecodeError:
-        with open(path, mode=mode, encoding='utf-8_sig') as f:
-            s = f.read()
-    # USTをノート単位に分割
-    l = [r'[#' + v.strip() for v in s.split(r'[#')][1:]
-    # さらに行ごとに分割して二次元リストに
-    l = [v.split('\n') for v in l]
-
-    # ノートのリストを作る
-    ust = Ust()
-    for lines in l:
-        note = Note()
-        # ノートの種類
-        tag = lines[0]
-        note.tag = tag
-        # print('Making "Note" instance from UST: {}'.format(tag))
-        # タグ以外の行の処理
-        if tag == '[#VERSION]':
-            note['UstVersion'] = lines[1]
-        elif tag == '[#TRACKEND]':
-            pass
-        else:
-            for line in lines[1:]:
-                key, value = line.split('=', 1)
-                note.set_by_key(key, value)
-        ust.append(note)
-
-    # 旧形式の場合にタグの数を合わせる
-    if ust[0].tag != '[#VERSION]':
-        try:
-            version = ust[0].get_by_key('UstVersion')
-        except KeyError:
-            print('WARN: USTファイルに [#VERSION] のエントリがありません。UTAU Ver 0.4.18 未満の場合はアップデートしてください。')
-            version = 'older_than_1.20'
-        note = Note()
-        note.tag = '[#VERSION]'
-        note.set_by_key('UstVersion', version)
-        ust.insert(0, note)  # リスト先頭に挿入
-    # UTAUプラグイン用のファイルとかで[#SETTING]がない場合にずれるのを対策
-    if ust[1].tag != '[#SETTING]':
-        note = Note()
-        note.tag = '[#SETTING]'
-        ust.insert(0, note)
-    # インスタンス変数に代入
-    ust.version = ust[0]
-    ust.setting = ust[1]
-    # 隠しパラメータ alternative_tempo を全ノートに設定
-    ust.reload_tempo()
-    return ust
+    new_ust = Ust()
+    new_ust.load(path, encoding=encoding)
+    return new_ust
 
 
-class Ust(UserList):
-    """UST"""
+class Ust:
+    """
+    UST (UTAU Sequence Text) ファイルを扱うためのクラス
+    """
 
     def __init__(self):
         super().__init__()
         # ノート(クラスオブジェクト)からなるリスト
-        self.version = None
-        self.setting = None
+        self.version = None   # [#VERSION]
+        self.setting = None   # [#SETTING]
+        self.notes: List[Note] = []       # [#1234], [#INSERT], [#DELETE]
+        self.trackend = None  # [#TRACKEND]
+        self.next_note = None      # [#NEXT]
+        self.previous_note = None      # [#PREV]
 
-    @property
-    def values(self):
-        """中身を見る"""
+    def __str__(self):
+        # self.notesが増減するので複製したものを扱う
+        duplicated_self = deepcopy(self)
+        # 特殊ノートを処理
+        if self.setting is not None:
+            duplicated_self.notes.insert(0, self.setting)
+        if self.previous_note is not None:
+            duplicated_self.notes.insert(0, self.previous_note)
+        if self.next_note is not None:
+            duplicated_self.notes.append(self.next_note)
+        if self.trackend is not None:
+            duplicated_self.notes.append(self.trackend)
+        # 通常ノートを文字列にする
+        s = '\n'.join(str(note) for note in duplicated_self.notes)
+        # バージョン情報があれば先頭に追記
+        if self.version is not None:
+            str_version = f'[#VERSION]\nUST Version {str(duplicated_self.version)}'
+            s = '\n'.join((str_version, s))
+        return s
+
+    def load(self, path: str, encoding='shift-jis'):
+        """
+        ファイルからインスタンス生成
+        """
+        # USTを文字列として取得
+        try:
+            with open(path, mode='r', encoding=encoding) as f:
+                s = f.read().strip()
+        except UnicodeDecodeError:
+            try:
+                with open(path, mode='r', encoding='utf-8') as f:
+                    s = f.read().strip()
+            except UnicodeDecodeError:
+                with open(path, mode='r', encoding='utf-8_sig') as f:
+                    s = f.read().strip()
+
+        # USTの文字列をノート単位に分割
+        l: List[str] = [f'[#{v.strip()}' for v in s.split('[#')][1:]
+        l_2d = [s.split('\n') for s in l]
+
+        # ノートのリストを作る
+        for lines in l_2d:
+            # 1行目: ノートの種類
+            tag = lines[0]
+            # print('reading entry:', tag)  # デバッグ用出力
+            note = Note(tag=tag)
+            # どこに登録するか決める
+            if tag not in ('[#VERSION]', '[#SETTING]', '[#TRACKEND]', '[#PREV]', '[#NEXT]'):
+                self.notes.append(note)
+            elif tag == '[#VERSION]':
+                self.version = lines[1].replace('UST Version ', '')
+                continue
+            elif tag == '[#SETTING]':
+                self.setting = note
+            elif tag == '[#PREV]':
+                self.previous_note = note
+            elif tag == '[#NEXT]':
+                self.next_note = note
+            elif tag == '[#TRACKEND]':
+                self.trackend = note
+            else:
+                raise Exception('想定外のエラーです。開発者に連絡してください。:', tag, str(note))
+            # 2行目移行: タグ以外の情報
+            for line in lines[1:]:
+                key, value = line.split('=', maxsplit=1)
+                note[key] = value
+
+        # 隠しパラメータ alternative_tempo を全ノートに設定
+        self.reload_tempo()
         return self
 
-    @values.setter
-    def values(self, l):
+    @ property
+    def tempo(self) -> float:
         """
-        中身を上書きする
-        テンポを正常に取得できるようにする
+        USTのグローバルテンポ
         """
-        if not isinstance(l, list):
-            raise TypeError('argument "l" must be list instance')
-        self.data = l
-        self.reload_tempo()
-        return self.data
+        # 2020-12-19以前の実装------------
+        # global_tempo = self.setting.get('Tempo', self.notes[0].tempo)
+        # --------------------------------
+        if 'Tempo' not in self.setting:
+            self.setting['Tempo'] = self.notes[0].tempo
+        return self.setting['Tempo']
 
-    @property
-    def notes(self) -> list:
-        """
-        全セクションのうち、[#VERSION] と [#SETTING] [#TRACKEND] を除いたノート部分を取得
-        """
-        return self.data[2:-1]
-
-    @notes.setter
-    def notes(self, l: list):
-        if not isinstance(l, list):
-            raise TypeError('argument "l" must be list instance')
-        self.data = self.data[:2] + l + self.data[-1:]
-        self.reload_tempo()
-        return self.data
-
-    @property
-    def tempo(self):
-        """全体のBPMを見る"""
-        try:
-            project_tempo = self.data[1].tempo
-            return project_tempo
-        except KeyError:
-            first_note_tempo = self.data[2].tempo
-            return first_note_tempo
-
-        print('[ERROR]--------------------------------------------------')
-        print('USTのテンポが設定されていません。とりあえず120にします。')
-        print('---------------------------------------------------------\n')
-        return '120'
-
-    @tempo.setter
+    @ tempo.setter
     def tempo(self, tempo):
-        """
-        グローバルBPMを上書きする
-        """
-        self.data[1].tempo = tempo
-        self.data[2].tempo = tempo
+        self.setting.tempo = tempo
+        # self.notes[0].tempo = tempo
         self.reload_tempo()
 
+    def clean_tempo(self):
+        """
+        ローカルテンポが不要な部分を削除する。
+        """
+        # まずはグローバルテンポを取得
+        current_tempo = self.tempo
+        # 不要なデータを削除
+        for note in self.notes:
+            if 'Tempo' in note:
+                if note['Tempo'] == current_tempo:
+                    del note['Tempo']
+            else:
+                current_tempo = note.tempo
+
     def reload_tempo(self):
         """
-        各ノートでBPMが取得できるように
-        独自パラメータ note.alternative_tempo を全ノートに仕込む
+        1. グローバルテンポを1ノート目のテンポで上書きする。
+        2. 各ノートでBPMが取得できるように note.alternative_tempo を全ノートに仕込む。
         """
-        current_tempo = self.tempo
+        if 'Tempo' in self.notes[0]:
+            self.setting['Tempo'] = self.notes[0]['Tempo']
+        current_tempo = self.setting['Tempo']
         for note in self.notes:
-            try:
-                current_tempo = note.get_by_key('Tempo')
-            except KeyError:
-                pass
+            if 'Tempo' in note:
+                if float(note['Tempo']) == current_tempo:
+                    del note['Tempo']
+                else:
+                    current_tempo = note['Tempo']
+            # current_tempo = note.get('Tempo', current_tempo)
             note.alternative_tempo = float(current_tempo)
+        self.clean_tempo()
 
-    def reload_tag_number(self):
+    def reload_tag_number(self, start: int = 0):
         """
+        start: 開始番号 (0なら[#0000]から)
         各ノートのノート番号を振りなおす。
         ファイル出力時に実行することを想定。
         """
-        for i, note in enumerate(self.notes):
+        for i, note in enumerate(self.notes, start):
             note.tag = f'[#{str(i).zfill(4)}]'
 
     # ノート一括編集系関数ここから----------------------------------------------
-    def replace_lyrics(self, before, after):
+    def replace_lyrics(self, before: str, after: str):
         """歌詞を一括置換（文字列指定・破壊的処理）"""
         for note in self.notes:
             note.lyric = note.lyric.replace(before, after)
@@ -210,7 +205,7 @@ class Ust(UserList):
             note.lyric = note.lyric.split()[-1]
     # ノート一括編集系関数ここまで----------------------------------------------
 
-    def insert_note(self, i):
+    def insert_note(self, i: int):
         """
         i 番目の区切りに新規ノートを挿入する。
         このときの i は音符のみのインデックス。
@@ -221,45 +216,36 @@ class Ust(UserList):
         self.notes.insert(i, note)
         return note
 
-    def delete_note(self, i):
+    def delete_note(self, i: int):
         """
         i 番目のノートを [#DELETE] する。
         """
         self.notes[i].tag = '[#DELETE]'
 
     def make_finalnote_R(self):
-        """Ustの最後のノートが必ず休符 になるようにする"""
-        note = self.data[-2]
+        """Ustの最後のノートが休符 になるようにする"""
+        last_note = self.notes[-1]
         # Ust内の最後はTRACKENDなので後ろから2番目のノートで判定
-        if note.lyric not in ('pau', 'sil', 'R'):
-            print('  末尾に休符を自動追加しました。')
-            extra_note = deepcopy(note)
-            extra_note.lyric = 'R'
-            extra_note.alternative_tempo = note.tempo
-            self.insert(-1, extra_note)
+        if last_note.lyric not in ('pau', 'sil', 'R'):
+            print('  末尾に休符を自動追加します。')
+            rest_note = deepcopy(last_note)
+            rest_note.lyric = 'R'
+            self.notes.append(rest_note)
+        self.reload_tempo()
 
-    def write(self, path: str, mode='w', encoding='shift-jis') -> str:
+    def write(self, path: str, mode: str = 'w', encoding: str = 'shift-jis') -> str:
         """
         USTをファイル出力
         """
         duplicated_self = deepcopy(self)
         # [#DELETE] なノートをファイル出力しないために削除
-        notes = [note for note in duplicated_self.notes if note.tag != '[#DELETE]']
-        duplicated_self.notes = notes
+        duplicated_self.notes = [
+            note for note in duplicated_self.notes if note.tag != '[#DELETE]'
+        ]
         # ノート番号を振りなおす
         duplicated_self.reload_tag_number()
-        # ノートのリストを文字列のリストに変換
-        lines = []
-        for note in duplicated_self:
-            if note.tag == '[#VERSION]':
-                lines.append('[#VERSION]')
-                lines.append(note['UstVersion'])
-                continue
-            lines.append(note.pop('Tag'))
-            for k, v in note.items():
-                lines.append(f'{str(k)}={str(v)}')
-        # 出力用の文字列
-        s = '\n'.join(lines)
+        # 文字列にする
+        s = str(duplicated_self) + '\n'
         # ファイル出力
         with open(path, mode=mode, encoding=encoding) as f:
             f.write(s)
@@ -269,29 +255,15 @@ class Ust(UserList):
 class Note(UserDict):
     """UST内のノート"""
 
-    def __init__(self, tag='[#UNDEFINED]'):
+    def __init__(self, tag: str = '[#UNDEFINED]'):
         super().__init__()
         self['Tag'] = tag
-        self.alternative_tempo = 120
+        self.alternative_tempo = None
 
     def __str__(self):
         lines = [self['Tag']] + [f'{k}={v}' for (k, v) in self.items() if k != 'Tag']
         return '\n'.join(lines)
 
-    @property
-    def values(self):
-        """ノートの中身を見る"""
-        return self.data
-
-    @values.setter
-    def values(self, d):
-        """
-        ノートの中身を上書き
-        """
-        if not isinstance(d, dict):
-            raise TypeError('argument "d" must be dictionary instance')
-        self.data = d
-
     @property
     def tag(self):
         """タグを確認"""
@@ -313,7 +285,7 @@ class Note(UserDict):
         self['Length'] = str(x)
 
     @property
-    def length_ms(self):
+    def length_ms(self) -> float:
         """ノート長を確認[ms]"""
         return 125 * float(self['Length']) / self.tempo
 
@@ -324,36 +296,36 @@ class Note(UserDict):
 
     @property
     def lyric(self):
-        """歌詞を確認"""
+        """
+        歌詞
+        """
         return self['Lyric']
 
     @lyric.setter
     def lyric(self, x):
-        """歌詞を上書き"""
         self['Lyric'] = x
 
     @property
     def notenum(self):
-        """音階番号を確認"""
+        """
+        音階番号
+        """
         return int(self['NoteNum'])
 
     @notenum.setter
     def notenum(self, x):
-        """音階番号を上書き"""
         self['NoteNum'] = str(x)
 
     @property
-    def tempo(self):
-        """ローカルBPMを取得"""
-        try:
-            return float(self['Tempo'])
-        except KeyError:
-            return float(self.alternative_tempo)
+    def tempo(self) -> float:
+        """
+        ローカルBPM
+        """
+        return float(self.get('Tempo', self.alternative_tempo))
 
     @tempo.setter
     def tempo(self, x):
-        """BPMを上書き"""
-        self['Tempo'] = x
+        self['Tempo'] = str(x)
 
     @property
     def pbs(self):
@@ -449,13 +421,29 @@ class Note(UserDict):
         str_pbm = ','.join(list_pbm)
         self['PBM'] = str_pbm
 
+    @property
+    def velocity(self):
+        """
+        子音速度
+        """
+        return int(self.get('Velocity', 100))
+
+    @velocity.setter
+    def velocity(self, x: int):
+        self['Velocity'] = int(x)
+
     # ここからデータ操作系-----------------------------------------------------
     def get_by_key(self, key):
-        """ノートの特定の情報を上書きまたは登録"""
+        """
+        ノートの特定の情報を取得
+        UserDictになったのでいずれ消す
+        """
         return self[key]
 
     def set_by_key(self, key, x):
-        """ノートの特定の情報を上書きまたは登録"""
+        """ノートの特定の情報を上書きまたは登録
+        UserDictになったのでいずれ消す
+        """
         self[key] = x
     # ここまでデータ操作系-----------------------------------------------------
 
@@ -480,27 +468,25 @@ class Note(UserDict):
 
     def suppin(self):
         """ノートの情報を最小限にする"""
-        new_note = {}
-        new_note['Tag'] = '[#DELETE]\n[#INSERT]'
-        new_note['Lyric'] = self.lyric
-        new_note['Length'] = self.length
-        new_note['NoteNum'] = self.notenum
-        self.data = new_note
+        new_data = {}
+        new_data['Tag'] = '[#DELETE]\n[#INSERT]'
+        new_data['Lyric'] = self.lyric
+        new_data['Length'] = self.length
+        new_data['NoteNum'] = self.notenum
+        self.data = new_data
     # ここまでノート操作系-----------------------------------------------------
 
-    # ここからデータ出力系-----------------------------------------------------
-    # Ustのほうで処理するようにしたので無効化しています。
-    # -------------------------------------------------------------------------
-    # def as_lines(self):
-    #     """出力用のリストを返す"""
-    #     d = self
-    #     lines = []
-    #     lines.append(d.pop('Tag'))
-    #     for k, v in d.items():
-    #         line = '{}={}'.format(str(k), str(v))
-    #         lines.append(line)
-    #     return lines
-    # ここまでデータ出力系-----------------------------------------------------
+
+def main():
+    """実行されたときの挙動"""
+    print('デフォ子かわいいよデフォ子\n')
+
+    print('ust読み取りテストをします。')
+    path = input('ustのパスを入力してください。\n>>> ')
+    ust = load(path)
+    print(ust)
+
+    input('\nPress Enter to exit.')
 
 
 if __name__ == '__main__':
